<?php

namespace Drupal\bookable_calendar\Plugin\Validation\Constraint;

use Drupal\Core\Entity\EntityPublishedInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Validates the CalendarOpeningVacancy constraint.
 */
class CalendarOpeningVacancyValidator extends ConstraintValidator {

  /**
   * {@inheritdoc}
   */
  public function validate($value, Constraint $constraint) {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $this->context->getRoot()->getValue();

    $parent_opening_instance = $entity->booking_instance->entity;
    $slots_available = $parent_opening_instance->slotsAvailable();
    $slots_as_parties = $parent_opening_instance->slotsAsParties();
    $slots_requested = (int) $entity->party_size->value;

    if ($slots_as_parties && $slots_available === 0) {
      $this->context->addViolation($constraint->noVacancy, [
        '%spots-requested' => 1,
        '%spots-available' => 0
      ]);
    }
    elseif ($slots_requested > $slots_available) {
      $this->context->addViolation($constraint->noVacancy, [
        '%spots-requested' => $slots_requested,
        '%spots-available' => $slots_available
      ]);
    }
  }

}
